"use strict";

exports.__esModule = true;
exports.default = exports.ContactsFeedsHandler = void 0;

var _rxjs = require("rxjs");

var _operators = require("rxjs/operators");

var _apiBzzBase = require("@erebos/api-bzz-base");

var _feed = require("../swarm/feed");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const MINUTE = 60 * 1000;
const DEFAULT_POLL_INTERVAL = 5 * MINUTE;

class FeedsHandler {
  constructor(context) {
    _defineProperty(this, "_context", void 0);

    _defineProperty(this, "_observers", new Set());

    _defineProperty(this, "_subscriptions", {});

    this._context = context;
  }

  remove(ref) {
    const sub = this._subscriptions[ref];

    if (sub != null) {
      sub.unsubscribe();
      delete this._subscriptions[ref];
    }
  }

  clear() {
    // $FlowFixMe: Object.values() losing type
    Object.values(this._subscriptions).forEach(sub => {
      sub.unsubscribe();
    });
    this._observers = new Set();
    this._subscriptions = {};
  }

}
/*
  There is a possible one-to-many relationship between a peer and contacts if
  the user has mutilple identities.
  The ContactsFeedsHandler handles this by multicasting and reference counting
  over a single observable for a given peer and adding one subscription per
  contact (cf https://rxjs.dev/guide/subject#reference-counting).
  It also keeps track of the observers (client subscriptions) in order to stop
  subscribing when there is no observer left.
  This logic would cause the peer feed polling to happen only if there is at
  least one contact linked to this peer, and the client subscribing to contacts
  changes. For every peer change, a single `peer_changed` event is emitted, and
  one `contact_changed` event for each contact linked to the peer.
*/


const peerSubscriptionRef = contact => `${contact.localID}-peerFeed`;

const contactSubscriptionRef = contact => `${contact.localID}-contactFeed`;

const contactFeedChangeSubscriptionRef = contact => `${contact.localID}-contactFeedChange`;

class ContactsFeedsHandler extends FeedsHandler {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "_peerFeeds", {});

    _defineProperty(this, "_peerDeletedSubscription", void 0);

    _defineProperty(this, "_contactCreatedSubscription", void 0);
  }

  _getPeerFeed(peer) {
    const existing = this._peerFeeds[peer.localID];

    if (existing != null) {
      return existing.refCounted;
    }

    const source = new _rxjs.Subject();
    const refCounted = (0, _feed.pollFeedJSON)(this._context.io.bzz, peer.publicFeed, {
      interval: DEFAULT_POLL_INTERVAL
    } // $FlowFixMe: ConnectableObservable pipe
    ).pipe((0, _operators.tap)(data => {
      // This mutation emits the `peer_changed` event
      this._context.mutations.updatePeerProfile(peer.localID, data.profile || {});
    }), (0, _operators.multicast)(source), (0, _operators.refCount)());
    this._peerFeeds[peer.localID] = {
      source,
      refCounted
    };
    return refCounted;
  }

  _subscribe(userID, contact) {
    if (!this._subscribePeerFeed(userID, contact)) return;
    if (!this._subscribeContactFeed(userID, contact)) return; // Delete subscription if contact gets deleted

    const sub = this._context.pipe((0, _operators.filter)(e => {
      return e.type === 'contact_deleted' && e.contactID === contact.localID;
    })).subscribe(() => {
      this._unsubscribe(contact);

      sub.unsubscribe();
    });
  }

  _subscribePeerFeed(userID, contact) {
    const peerSubRef = peerSubscriptionRef(contact);

    if (this._subscriptions[peerSubRef] != null) {
      return false;
    }

    const peer = this._context.openVault.identities.getPeerUser(contact.peerID);

    if (peer == null) {
      this._context.log('No peer found for contact', contact);

      return false;
    }

    this._subscriptions[peerSubRef] = this._getPeerFeed(peer).subscribe({
      next: () => {
        // Emit the `contact_changed` event so the profile mutation on the peer data can be observed by the subscription
        this._context.next({
          type: 'contact_changed',
          contact,
          userID,
          change: 'peerChanged'
        });
      },
      error: err => {
        this._context.log('peer feed subscription error', err);
      },
      complete: () => {
        delete this._subscriptions[peerSubRef];
      }
    });
    return true;
  }

  _subscribeContactFeed(userID, contact) {
    const contactSubRef = contactSubscriptionRef(contact);
    const feedChangeSubRef = contactFeedChangeSubscriptionRef(contact);

    if (this._subscriptions[contactSubRef] != null) {
      return false;
    }

    const user = this._context.openVault.identities.getOwnUser(userID);

    const peer = this._context.openVault.identities.getPeerUser(contact.peerID);

    if (user == null || peer == null) {
      this._context.log(`No ${user == null ? 'user' : 'peer'} found for contact`, contact);

      return false;
    } // Resubscribe if feedHash changes


    if (this._subscriptions[feedChangeSubRef] == null) {
      this._subscriptions[feedChangeSubRef] = this._context.pipe((0, _operators.filter)(e => {
        return e.type === 'contact_changed' && e.contact.localID === contact.localID && e.change === 'remoteFeed';
      })).subscribe(() => {
        this.remove(contactSubRef);

        const res = this._subscribeContactFeed(userID, contact);

        this._context.log('contact feed resubscription:', res);
      });
    }

    if (contact.connectionState !== 'connected') {
      const topic = (0, _apiBzzBase.getFeedTopic)({
        name: user.base64PublicKey()
      });
      this._subscriptions[contactSubRef] = this._context.io.bzz.pollFeedValue(peer.firstContactAddress, {
        mode: 'content-response',
        whenEmpty: 'ignore',
        contentChangedOnly: true,
        immediate: true,
        interval: DEFAULT_POLL_INTERVAL
      }, {
        topic
      }).pipe((0, _operators.flatMap)(res => res.json())).subscribe({
        next: data => {
          // TODO: validate payload version
          this._context.mutations.setContactFeed(userID, contact.localID, data.privateFeed);
        },
        error: err => {
          this._context.log('contact feed subscription error', err);
        },
        complete: () => {
          delete this._subscriptions[contactSubRef];
        }
      });
    } else {
      this._subscriptions[contactSubRef] = contact.sharedFeed.pollRemoteData(this._context.io.bzz, {
        interval: DEFAULT_POLL_INTERVAL
      }).subscribe({
        next: data => {
          // TODO: validate payload version
          if (data.profile != null) {
            this._context.mutations.updateContactProfile(userID, contact.localID, data.profile);
          }

          if (data.apps != null) {
            this._context.mutations.updateContactApps(userID, contact.localID, data.apps);
          }
        },
        error: err => {
          this._context.log('contact feed subscription error', err);
        },
        complete: () => {
          delete this._subscriptions[contactSubRef];
        }
      });
    }

    return true;
  }

  _unsubscribe(contact) {
    this.remove(peerSubscriptionRef(contact));
    this.remove(contactSubscriptionRef(contact));
    this.remove(contactFeedChangeSubscriptionRef(contact));
  }

  _setup() {
    // Subscribe to peer deleted event to stop polling its feed
    this._peerDeletedSubscription = this._context.pipe((0, _operators.filter)(e => e.type === 'peer_deleted')).subscribe(e => {
      const feed = this._peerFeeds[e.peerID];

      if (feed != null) {
        feed.source.unsubscribe();
        delete this._peerFeeds[e.peerID];
      }
    }); // Subscribe to contact created event to start polling new contacts

    this._contactCreatedSubscription = this._context.pipe((0, _operators.filter)(e => e.type === 'contact_created')).subscribe(e => {
      this.add(e.userID, e.contact);
    }); // Subscribe to all contacts

    const {
      contacts
    } = this._context.openVault.identities;
    Object.keys(contacts).forEach(userID => {
      // $FlowFixMe: Object.values() losing type
      Object.values(contacts[userID]).forEach(contact => {
        this._subscribe(userID, contact);
      });
    });
  }

  add(userID, contact) {
    // Only add subscription for the contact if
    if (this._observers.size > 0) {
      this._subscribe(userID, contact);
    }
  }

  observe() {
    if (this._observers.size === 0) {
      this._setup();
    }

    const source = this._context.pipe((0, _operators.filter)(e => e.type === 'contact_changed'));

    this._observers.add(source);

    return {
      dispose: () => {
        this._observers.delete(source);

        if (this._observers.size === 0) {
          this.clear();
        }
      },
      source
    };
  }

  clear() {
    super.clear();

    if (this._peerDeletedSubscription != null) {
      this._peerDeletedSubscription.unsubscribe();
    }

    if (this._contactCreatedSubscription != null) {
      this._contactCreatedSubscription.unsubscribe();
    }
  }

}

exports.ContactsFeedsHandler = ContactsFeedsHandler;
var _default = FeedsHandler;
exports.default = _default;