"use strict";

exports.__esModule = true;
exports.BidirectionalFeed = exports.pollFeedJSON = exports.fetchJSON = exports.OwnFeed = exports.randomTopic = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _apiBzzNode = _interopRequireDefault(require("@erebos/api-bzz-node"));

var _apiBzzBase = require("@erebos/api-bzz-base");

var _hex = require("@erebos/hex");

var _secp256k = require("@erebos/secp256k1");

var _keccak = require("@erebos/keccak256");

var _operators = require("rxjs/operators");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const randomTopic = () => {
  return (0, _hex.hexValueType)('0x' + _crypto.default.randomBytes(32).toString('hex'));
};

exports.randomTopic = randomTopic;

class OwnFeed {
  constructor(keyPair, topic, feedHash) {
    _defineProperty(this, "_keyPair", void 0);

    _defineProperty(this, "_address", void 0);

    _defineProperty(this, "_topic", void 0);

    _defineProperty(this, "_feedHash", void 0);

    this._keyPair = keyPair;
    this._topic = topic;
    this._feedHash = feedHash;
  }

  get keyPair() {
    return this._keyPair;
  }

  get address() {
    if (this._address == null) {
      this._address = (0, _keccak.pubKeyToAddress)(this._keyPair.getPublic().encode());
    }

    return this._address;
  }

  get topic() {
    return this._topic;
  }

  get feedHash() {
    return this._feedHash;
  }

  async syncManifest(bzz) {
    if (this.feedHash) return false;
    const feedHash = await bzz.createFeedManifest(this.address, {
      topic: this.topic
    });
    this._feedHash = feedHash;
    return true;
  }

  async publishJSON(bzz, payload) {
    return await bzz.uploadFeedValue(this.address, JSON.stringify(payload), {
      topic: this.topic
    }, {
      contentType: 'application/json'
    }, this.keyPair.getPrivate());
  }

}

exports.OwnFeed = OwnFeed;

_defineProperty(OwnFeed, "create", (keyPair, topicOrName) => {
  let topic = topicOrName;

  if (!(0, _hex.isHexValue)(topicOrName)) {
    topic = (0, _apiBzzBase.getFeedTopic)({
      name: topicOrName
    });
  }

  return new OwnFeed(keyPair || (0, _secp256k.createKeyPair)(), (0, _hex.hexValueType)(topic));
});

_defineProperty(OwnFeed, "fromJSON", serialized => {
  return new OwnFeed((0, _secp256k.createKeyPair)(serialized.ethPrivKey, 'hex'), serialized.topic, serialized.feedHash);
});

_defineProperty(OwnFeed, "toJSON", feed => {
  return {
    ethPrivKey: feed.keyPair.getPrivate('hex'),
    topic: feed.topic,
    feedHash: feed.feedHash
  };
});

const fetchJSON = (bzz, hash, options) => {
  return bzz.download(hash, options).then(res => res.json());
};

exports.fetchJSON = fetchJSON;

const pollFeedJSON = (bzz, hash, options) => {
  return bzz.pollFeedValue(hash, _objectSpread({
    mode: 'content-response',
    whenEmpty: 'ignore',
    contentChangedOnly: true
  }, options)).pipe((0, _operators.flatMap)(res => res.json()));
};

exports.pollFeedJSON = pollFeedJSON;

class BidirectionalFeed {
  constructor(localFeed, remoteFeed, localFeedData) {
    _defineProperty(this, "_localFeed", void 0);

    _defineProperty(this, "_localFeedData", void 0);

    _defineProperty(this, "_remoteFeed", void 0);

    this._localFeed = localFeed;
    this._localFeedData = localFeedData;
    this._remoteFeed = remoteFeed;
  }

  get localFeed() {
    return this._localFeed;
  } // Lazily read this from local feed instead of serializing?


  get localFeedData() {
    return this._localFeedData || {};
  }

  set localFeedData(data) {
    this._localFeedData = data;
  }

  get remoteFeed() {
    return this._remoteFeed;
  }

  set remoteFeed(remoteFeed) {
    this._remoteFeed = remoteFeed;
  }

  async syncManifest(bzz) {
    return this.localFeed.syncManifest(bzz);
  }

  async publish(bzz, data) {
    return this.localFeed.publishJSON(bzz, data);
  }

  async publishLocalData(bzz) {
    return this.publish(bzz, this.localFeedData);
  }

  pollRemoteData(bzz, options) {
    if (this.remoteFeed == null) {
      throw new Error('Failed to poll remote feed, hash unknown');
    }

    return pollFeedJSON(bzz, this.remoteFeed, options);
  }

}

exports.BidirectionalFeed = BidirectionalFeed;

_defineProperty(BidirectionalFeed, "create", optional => {
  return new BidirectionalFeed(optional && optional.localFeed || OwnFeed.create(undefined, randomTopic()), optional && optional.remoteFeed);
});

_defineProperty(BidirectionalFeed, "fromJSON", serialized => new BidirectionalFeed(OwnFeed.fromJSON(serialized.localFeed), serialized.remoteFeed, serialized.localFeedData));

_defineProperty(BidirectionalFeed, "toJSON", bidirectionalFeed => ({
  localFeed: OwnFeed.toJSON(bidirectionalFeed.localFeed),
  localFeedData: bidirectionalFeed._localFeedData,
  remoteFeed: bidirectionalFeed.remoteFeed
}));