"use strict";

exports.__esModule = true;
exports.default = exports.readVaultFile = exports.createVaultKeyParams = void 0;

var _secureFile = require("@mainframe/secure-file");

var _utilsBase = require("@mainframe/utils-base64");

var _utilsCrypto = require("@mainframe/utils-crypto");

var _eth = require("@mainframe/eth");

var _AppsRepository = _interopRequireDefault(require("../app/AppsRepository"));

var _IdentitiesRepository = _interopRequireDefault(require("../identity/IdentitiesRepository"));

var _WalletsRepository = _interopRequireDefault(require("../wallet/WalletsRepository"));

var _IdentityWallets = _interopRequireDefault(require("../identity/IdentityWallets"));

var _ContactAppData = _interopRequireDefault(require("../contact/ContactAppData"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const createVaultKeyParams = async password => {
  const salt = (0, _utilsCrypto.createPasswordHashSalt)();
  const kdf = {
    algorithm: _utilsCrypto.PASSWORDHASH_ALG_ARGON2ID13,
    memlimit: _utilsCrypto.PASSWORDHASH_MEMLIMIT_SENSITIVE,
    opslimit: _utilsCrypto.PASSWORDHASH_OPSLIMIT_SENSITIVE,
    salt: (0, _utilsBase.encodeBase64)(salt)
  };
  const key = await (0, _utilsCrypto.createSecretBoxKeyFromPassword)(password, salt, kdf.opslimit, kdf.memlimit, kdf.algorithm);
  return {
    kdf,
    key
  };
};

exports.createVaultKeyParams = createVaultKeyParams;

const readVaultFile = async (path, password) => {
  let keyParams;
  const file = await (0, _secureFile.readEncryptedFile)(path, async meta => {
    if (meta == null) {
      throw new Error('Missing metadata');
    }

    if (meta.version !== 1) {
      throw new Error('Invalid vault format version');
    }

    if (meta.kdf == null) {
      throw new Error('Missing KDF parameters in metadata');
    }

    const key = await (0, _utilsCrypto.createSecretBoxKeyFromPassword)(password, (0, _utilsBase.decodeBase64)(meta.kdf.salt), meta.kdf.opslimit, meta.kdf.memlimit, meta.kdf.algorithm);
    keyParams = {
      key,
      kdf: meta.kdf
    };
    return key;
  });

  if (file.opened == null) {
    throw new Error('Invalid password');
  }

  if (keyParams == null) {
    throw new Error('Invalid file');
  }

  return {
    data: JSON.parse(file.opened.toString()),
    keyParams
  };
};

exports.readVaultFile = readVaultFile;

class Vault {
  constructor(path, keyParams, data) {
    _defineProperty(this, "_path", void 0);

    _defineProperty(this, "_keyParams", void 0);

    _defineProperty(this, "_data", void 0);

    _defineProperty(this, "_sessions", {});

    this._path = path;
    this._keyParams = keyParams;
    const vaultData = {
      apps: new _AppsRepository.default(),
      identities: new _IdentitiesRepository.default(),
      settings: {
        bzzURL: 'http://mainframe-gateways.net:8500',
        pssURL: 'ws://mainframe-gateways.net:8546',
        ethURL: _eth.INFURA_URLS.mainnet
      },
      identityWallets: new _IdentityWallets.default(),
      contactAppData: new _ContactAppData.default(),
      wallets: new _WalletsRepository.default()
    };
    this._data = data ? Object.assign(vaultData, data) : vaultData;
  } // Getters


  get path() {
    return this._path;
  }

  get apps() {
    return this._data.apps;
  }

  get identities() {
    return this._data.identities;
  }

  get settings() {
    return this._data.settings;
  }

  get wallets() {
    return this._data.wallets;
  }

  get identityWallets() {
    return this._data.identityWallets;
  }

  get contactAppData() {
    return this._data.contactAppData;
  } // App lifecycle


  closeApp(sessID) {
    delete this._sessions[sessID];
  }

  openApp(appID, userID) {
    const sessionData = this.apps.createSession(appID, userID);
    this._sessions[sessionData.sessID] = sessionData.session;
    return sessionData;
  }

  installApp(manifest, userID, settings) {
    let app = this.apps.getByMFID(manifest.id);

    if (app == null) {
      // Add app with user settings
      app = this.apps.add(manifest, userID, settings);
    } else {
      // Set user settings for already existing app
      this.apps.setUserPermissionsSettings(app.id, userID, settings);
    }

    return app;
  }

  removeApp(appID) {
    this.apps.remove(appID);
  }

  getSession(id) {
    return this._sessions[id];
  } // App creation and management


  setAppPermissionsRequirements(appID, permissions, version) {
    const app = this.apps.getOwnByID(appID);

    if (app == null) {
      throw new Error('App not found');
    }

    app.setPermissionsRequirements(permissions, version);
  }

  setAppUserPermissionsSettings(appID, userID, settings) {
    this.apps.setUserPermissionsSettings(appID, userID, settings);
  } // Settings


  setSettings(settings) {
    Object.assign(this._data.settings, settings);
    return this._data.settings;
  }

  setBzzURL(url) {
    this._data.settings.bzzURL = url;
  }

  setPssURL(url) {
    this._data.settings.pssURL = url;
  }

  setEthUrl(url) {
    this._data.settings.ethURL = url;
  } // Vault lifecycle


  save() {
    const data = Buffer.from(JSON.stringify(this.toJSON()));
    return (0, _secureFile.writeEncryptedFile)(this._path, data, this._keyParams.key, {
      version: 1,
      kdf: this._keyParams.kdf
    });
  }

  toJSON() {
    return this._data ? {
      apps: _AppsRepository.default.toJSON(this._data.apps),
      identities: _IdentitiesRepository.default.toJSON(this._data.identities),
      wallets: _WalletsRepository.default.toJSON(this._data.wallets),
      identityWallets: _IdentityWallets.default.toJSON(this._data.identityWallets),
      contactAppData: _ContactAppData.default.toJSON(this._data.contactAppData),
      settings: this._data.settings
    } : {};
  }

}

exports.default = Vault;

_defineProperty(Vault, "create", async (path, password) => {
  const keyParams = await createVaultKeyParams(password);
  const vault = new Vault(path, keyParams);
  await vault.save();
  return vault;
});

_defineProperty(Vault, "open", async (path, password) => {
  const {
    data,
    keyParams
  } = await readVaultFile(path, password);
  return new Vault(path, keyParams, {
    apps: _AppsRepository.default.fromJSON(data.apps),
    identities: _IdentitiesRepository.default.fromJSON(data.identities),
    wallets: _WalletsRepository.default.fromJSON(data.wallets),
    identityWallets: _IdentityWallets.default.fromJSON(data.identityWallets),
    contactAppData: _ContactAppData.default.fromJSON(data.contactAppData),
    settings: data.settings
  });
});