"use strict";

exports.__esModule = true;
exports.default = void 0;

var _utilsId = require("@mainframe/utils-id");

var _utils = require("../utils");

var _HDWallet = _interopRequireDefault(require("./HDWallet"));

var _LedgerWallet = _interopRequireDefault(require("./LedgerWallet"));

var _ledgerClient = require("./ledgerClient");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const hdWalletsToJSON = (0, _utils.mapObject)(_HDWallet.default.toJSON);
const hdWalletsFromJSON = (0, _utils.mapObject)(_HDWallet.default.fromJSON);
const ledgerWalletsToJSON = (0, _utils.mapObject)(_LedgerWallet.default.toJSON);
const ledgerWalletsFromJSON = (0, _utils.mapObject)(_LedgerWallet.default.fromJSON);

class WalletsRepository {
  constructor(params) {
    _defineProperty(this, "_wallets", void 0);

    if (params) {
      this._wallets = params.wallets;
    } else {
      this._wallets = {
        ethereum: {
          hd: {},
          ledger: {}
        }
      };
    }
  }

  get ethWallets() {
    return this._wallets.ethereum;
  }

  getEthWalletByID(id) {
    return this.ethWallets.hd[id] || this.ethWallets.ledger[id];
  }

  getEthHDWallet(id) {
    return this.ethWallets.hd[id];
  }

  getEthLedgerWallet(id) {
    return this.ethWallets.ledger[id];
  }

  getFirstEthWallet() {
    const priority = ['ledger', 'hd'];
    let wallet;
    priority.some(type => {
      if (Object.keys(this.ethWallets[type]).length) {
        wallet = this.ethWallets[type][Object.keys(this.ethWallets[type])[0]];
        return true;
      }

      return false;
    });
    return wallet;
  }

  getEthWalletByAccount(account) {
    let wallet;
    Object.keys(this.ethWallets).forEach(type => {
      Object.keys(this.ethWallets[type]).forEach(w => {
        const containingWallet = this.ethWallets[type][w];
        const accountWallet = containingWallet.containsAccount(account);

        if (accountWallet) {
          wallet = containingWallet;
        }
      });
    });
    return wallet;
  }

  createHDWallet(blockchain, name) {
    switch (blockchain) {
      case 'ethereum':
        {
          const hdWallet = _HDWallet.default.create(name);

          this.ethWallets.hd[hdWallet.id] = hdWallet;
          return hdWallet;
        }

      default:
        {
          throw new Error(`Unsupported blockchain type: ${blockchain}`);
        }
    }
  }

  importMnemonicWallet(params) {
    switch (params.blockchain) {
      case 'ethereum':
        {
          const alreadyExists = Object.keys(this.ethWallets.hd).find(id => {
            return this.ethWallets.hd[id].mnemonic === params.mnemonic;
          });

          if (alreadyExists) {
            throw new Error('Wallet with this mnemonic phrase already exists');
          }

          const hdWallet = _HDWallet.default.import({
            mnemonic: params.mnemonic,
            name: params.name
          });

          this.ethWallets.hd[hdWallet.id] = hdWallet;
          return hdWallet;
        }

      default:
        {
          throw new Error(`Unsupported blockchain type: ${params.blockchain}`);
        }
    }
  }

  addHDWalletAccount(params) {
    const wallet = this.getEthHDWallet(params.walletID);

    if (!wallet) {
      throw new Error('Wallet not found');
    }

    const addresses = wallet.addAccounts([params.index]);
    return addresses[0];
  }

  deleteWallet(chain, type, walletID) {
    if (!this._wallets[chain]) {
      throw new Error(`Unsupported chain: ${chain}`);
    }

    if (!this._wallets[chain][type]) {
      throw new Error(`Invalid wallet type: ${type}`);
    }

    const wallet = this._wallets[chain][type][walletID];

    if (!wallet) {
      throw new Error('Wallet not found');
    }

    delete this._wallets[chain][type][walletID];
  } // Signing


  async signTransaction(blockchain, txParams) {
    switch (blockchain) {
      case 'ethereum':
        return this.signEthTransaction(txParams);

      default:
        throw new Error(`Unsupported blockchain type: ${blockchain}`);
    }
  }

  async signEthTransaction(params) {
    // TODO: set chain ID
    const wallet = this.getEthWalletByAccount(params.from);

    if (!wallet) {
      throw new Error('Wallet not found');
    }

    return wallet.signTransaction(params);
  } // Ledger Wallets


  getLedgerEthWalletByFirstAddr(address) {
    const ledgerWallets = this._wallets.ethereum.ledger;
    const walletID = Object.keys(ledgerWallets).find(w => {
      return ledgerWallets[w]._firstAddress === address;
    });

    if (walletID) {
      return this._wallets.ethereum.ledger[walletID];
    }
  }

  async addLedgerEthAccounts(indexes, name) {
    // Identify ledger wallets with first address
    const firstLedgerAddr = await (0, _ledgerClient.getAddressAtIndex)({
      index: 0
    });
    let ledgerWallet = this.getLedgerEthWalletByFirstAddr(firstLedgerAddr);

    if (!ledgerWallet) {
      const walletID = (0, _utilsId.uniqueID)();
      ledgerWallet = new _LedgerWallet.default({
        name,
        localID: walletID,
        firstAddress: firstLedgerAddr,
        activeAccounts: {}
      });
      this._wallets.ethereum.ledger[walletID] = ledgerWallet;
    }

    const addresses = await ledgerWallet.addAccounts(indexes);
    return {
      localID: ledgerWallet.id,
      addresses: addresses
    };
  }

}

exports.default = WalletsRepository;

_defineProperty(WalletsRepository, "fromJSON", serialized => {
  return new WalletsRepository({
    wallets: {
      ethereum: {
        // $FlowFixMe: mapping type
        hd: hdWalletsFromJSON(serialized.wallets.ethereum.hd),
        // $FlowFixMe: mapping type
        ledger: ledgerWalletsFromJSON(serialized.wallets.ethereum.ledger)
      }
    }
  });
});

_defineProperty(WalletsRepository, "toJSON", registry => {
  return {
    wallets: {
      ethereum: {
        // $FlowFixMe: mapping type
        hd: hdWalletsToJSON(registry.ethWallets.hd),
        // $FlowFixMe: mapping type
        ledger: ledgerWalletsToJSON(registry.ethWallets.ledger)
      }
    }
  };
});